import uuid

from typing import List

import barker.schemas.device as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.device import Device
from ..models.login_history import LoginHistory
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.put("/{id_}", response_model=schemas.Device)
def update(
    id_: uuid.UUID,
    data: schemas.DeviceIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["devices"]),
) -> schemas.Device:
    try:
        item: Device = db.query(Device).filter(Device.id == id_).first()
        item.name = data.name
        item.enabled = data.enabled
        item.section_id = data.section.id_
        db.commit()
        return device_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}")
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["add-devices"]),
):
    try:
        db.execute(LoginHistory.__table__.delete(LoginHistory.device_id == id_))
        db.execute(Device.__table__.delete(Device.id == id_))
        db.commit()
        return {}
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("/list", response_model=List[schemas.Device])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [device_info(item) for item in db.query(Device).order_by(Device.name).all()]


@router.get("/{id_}", response_model=schemas.Device)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["devices"]),
) -> schemas.Device:
    item: Device = db.query(Device).filter(Device.id == id_).first()
    return device_info(item)


def device_info(item: Device) -> schemas.Device:
    return schemas.Device(
        id=item.id,
        name=item.name,
        enabled=item.enabled,
        section={"id": item.section_id, "name": item.section.name},
        creationDate=item.creation_date,
        lastUser=item.login_history[0].user.name if len(item.login_history) else "Never",
        lastDate=item.login_history[0].date if len(item.login_history) else None,
    )
