import barker.schemas.header_footer as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.db_setting import DbSetting
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.get("")
def get(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["owner"]),
):
    return [
        {"id": item.id, "name": item.name, "text": item.data["Text"]}
        for item in db.query(DbSetting).filter(DbSetting.name.in_(["Header", "Footer"])).all()
    ]


@router.post("")
def save(
    data: schemas.HeaderFooter,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["owner"]),
):
    try:
        item: DbSetting = db.query(DbSetting).filter(DbSetting.id == data.id_, DbSetting.name == data.name).first()
        item.data = {"Text": data.text}
        db.commit()
        return [
            {"id": item.id, "name": item.name, "text": item.data["Text"]}
            for item in db.query(DbSetting).filter(DbSetting.name.in_(["Header", "Footer"])).all()
        ]
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise
