import uuid

from typing import List

import barker.schemas.section as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.section import Section
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.Section)
def save(
    data: schemas.SectionIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["sections"]),
) -> schemas.Section:
    try:
        item = Section(name=data.name)
        db.add(item)
        db.commit()
        return section_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.Section)
def update(
    id_: uuid.UUID,
    data: schemas.SectionIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["sections"]),
) -> schemas.Section:
    try:
        item: Section = db.query(Section).filter(Section.id == id_).first()
        if item.is_fixture:
            raise HTTPException(
                status_code=status.HTTP_423_LOCKED,
                detail=f"{item.name} is a fixture and cannot be edited or deleted.",
            )
        item.name = data.name
        db.commit()
        return section_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.SectionBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["sections"]),
) -> schemas.SectionBlank:
    try:
        item: Section = db.query(Section).filter(Section.id == id_).first()
        if item.is_fixture:
            raise HTTPException(
                status_code=status.HTTP_423_LOCKED,
                detail=f"{item.name} is a fixture and cannot be edited or deleted.",
            )
        db.delete(item)
        db.commit()
        return section_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.SectionBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["sections"]),
) -> schemas.SectionBlank:
    return section_blank()


@router.get("/list", response_model=List[schemas.Section])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [section_info(item) for item in db.query(Section).order_by(Section.name).all()]


@router.get("/{id_}", response_model=schemas.Section)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["sections"]),
) -> schemas.Section:
    item: Section = db.query(Section).filter(Section.id == id_).first()
    return section_info(item)


def section_info(item: Section) -> schemas.Section:
    return schemas.Section(id=item.id, name=item.name)


def section_blank() -> schemas.SectionBlank:
    return schemas.SectionBlank(name="")
