import uuid

from typing import List, Optional

import barker.schemas.section_printer as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy import and_, or_
from sqlalchemy.dialects.postgresql import insert as pg_insert
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.menu_category import MenuCategory
from ..models.section_printer import SectionPrinter
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("/{id_}", response_model=List[schemas.SectionPrinter])
def save(
    id_: uuid.UUID,
    data: List[schemas.SectionPrinter],
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["section-printers"]),
) -> List[schemas.SectionPrinter]:
    try:
        current = []
        default_copies = 0
        for mcs in data:
            if mcs.menu_category is None and mcs.printer is None:
                raise ValueError("Please choose a default printer")
            if mcs.printer is None:
                continue
            if mcs.menu_category is None:
                default_copies = mcs.copies
            stmt = (
                pg_insert(SectionPrinter.__table__)
                .values(
                    id=uuid.uuid4(),
                    menu_category_id=mcs.menu_category.id_ if mcs.menu_category is not None else None,
                    section_id=id_,
                    printer_id=mcs.printer.id_,
                    copies=mcs.copies,
                )
                .on_conflict_do_update(
                    index_elements=["menu_category_id", "section_id"],
                    set_=dict(printer_id=mcs.printer.id_, copies=mcs.copies),
                )
            )
            db.execute(stmt)
            current.append(None if mcs.menu_category is None else mcs.menu_category.id_)
        db.execute(
            SectionPrinter.__table__.delete(
                or_(
                    and_(
                        SectionPrinter.section_id == id_,
                        SectionPrinter.menu_category_id == None,  # noqa: E711
                        SectionPrinter.copies != default_copies,
                    ),
                    and_(
                        SectionPrinter.section_id == id_,
                        SectionPrinter.menu_category_id != None,  # noqa: E711
                        ~SectionPrinter.menu_category_id.in_([x for x in current if x is not None]),
                    ),
                )
            )
        )
        db.commit()

        return report(id_, db)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=List[schemas.SectionPrinter])
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["section-printers"]),
) -> List[schemas.SectionPrinter]:
    try:
        SectionPrinter.__table__.delete(SectionPrinter.section_id == id_)
        db.commit()
        return report(id_, db)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=List[schemas.SectionPrinter])
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["section-printers"]),
) -> List[schemas.SectionPrinter]:
    return report(None, db)


@router.get("/{id_}", response_model=List[schemas.SectionPrinter])
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["section-printers"]),
) -> List[schemas.SectionPrinter]:
    return report(id_, db)


def report(section_id: Optional[uuid.UUID], db: Session) -> List[schemas.SectionPrinter]:
    menu_categories = (
        db.query(MenuCategory.id, MenuCategory.name)
        .filter(MenuCategory.is_active == True)  # noqa: E712
        .order_by(MenuCategory.sort_order)
        .all()
    )
    list_ = []
    for mc_id, mc_name in [(None, None)] + menu_categories:
        section_printer = (
            db.query(SectionPrinter)
            .filter(
                SectionPrinter.section_id == section_id,
                SectionPrinter.menu_category_id == mc_id,
            )
            .first()
        )
        list_.append(
            schemas.SectionPrinter(
                menuCategory=None if mc_id is None else schemas.MenuCategoryLink(id=mc_id, name=mc_name),
                printer=None if section_printer is None else schemas.PrinterLink(id=section_printer.printer_id),
                copies=0 if section_printer is None else section_printer.copies,
            )
        )
    return list_
